import Runtime from './Runtime'
import Debugger from './Debugger'
import RuntimeIns from './lib/RuntimeIns'
import DebuggerIns from './lib/DebuggerIns'
import timer from './util/timer'
import util from './util'

if (!window.finclip_rins) {
  window.finclip_rins = new RuntimeIns()
  window.finclip_dins = new DebuggerIns()

  window.finclip_runtime = null

  window.finclip_debug_warn_log = function () {
    console.warn('请优化代码结构, 便于调试')
  }

  window.finclip_debug_before = function (data, arg) {
    window.finclip_dins.debuggerObj.oldData = data
  }

  window.finclip_debug_after = function (scriptId, line, funName) {
    const { oldData, breakpoint, callFrame } = window.finclip_dins.debuggerObj
    // if (debuggerObj.nowScriptId !== scriptId) {
    //   debuggerObj.callFrame = []
    // }
    const item = breakpoint.find((e) => {
      return e.scriptId == scriptId && line === e.line
    })
    if (funName.name !== '') {
      window.finclip_dins.debuggerObj.callFrame.push(funName.name)
    }
    if (item) {
      window.fincli_debuggerins.debuggerFirePausedEvent({
        res: oldData,
        callFrame: callFrame,
        t: item.start
      })
    }
    window.finclip_dins.debuggerObj.nowScriptId = scriptId
  }

  // [基础库 -> sdk] -> remote
  const sendMsg = (target, payload) => {
    window.remoteDebugger && window.remoteDebugger.remoteDebugger({
      target,
      data: payload
    })
  }

  const ws = {
    send: function (msg) {
      sendMsg('runtime', msg)
    }
  }

  window.finclip_runtime = new Runtime(ws, window.finclip_rins, window.finclip_dins)
  timer.init(window.finclip_runtime)
  window.fincli_debuggerins = null

  const handleRuntimeCDP = (payload) => {
    const params = {
      id: payload.id,
      method: payload.method || '',
      params: payload.params
    }
    if (params.method.startsWith('Runtime.')) {
      window.finclip_runtime.handle(params)
    }
    if (params.method.startsWith('Debugger.')) {
      if (!window.fincli_debuggerins) window.fincli_debuggerins = new Debugger(ws, window.finclip_rins, window.finclip_dins)
      window.fincli_debuggerins.handle(params)
    }
  }

  let currentPage
  setTimeout(() => {
    window.execOnServiceReady(() => {
      wx.onAppRoute(function (params) {
        currentPage = params
        sendMsg('dom', { method: 'DOM.documentUpdated' })
      })
    })
  }, 0)

  const handleDomCDP = (payload) => {
    // Dom
    window.ServiceJSBridge.publish('remoteDebugger', payload, [currentPage.webviewId])
  }

  // sdk -> 基础库
  window.ServiceJSBridge.on('remoteDebugger', (msg) => {
    if (msg.target === 'runtime') {
      handleRuntimeCDP(msg.data)
      return
    }
    if (msg.target === 'dom') {
      handleDomCDP(msg.data)
      return
    }
    if (msg.data.id) {
      sendMsg(msg.target, {
        id: msg.data.id,
        result: {}
      })
    }
  })

  window.iosRemoteDebuggerLog = function iosRemoteDebuggerLog (data) {
    if (!window.finclip_runtime || !util.isIOS) {
      return
    }
    if (window.finclip_runtime.enable) {
      window.finclip_runtime.onConsoleOutput(data)
    } else {
      window.finclip_runtime.setLog(data)
    }
  }
}
