function getSpecialType (element) {
  if (element instanceof Map) {
    return 'Map'
  }
  if (element instanceof Set) {
    return 'Set'
  }
  if (element instanceof ArrayBuffer) {
    return 'ArrayBuffer'
  }
  if (element && element.byteLength) {
    return 'ArrayBuffer'
  }
  return false
}

// 不支持对象统一返回空对象
function unsupportedObjects (obj) {
  let tag = false
  if (obj instanceof WeakMap) {
    tag = true
  }
  if (obj instanceof WeakSet) {
    tag = true
  }
  if (obj instanceof Promise) {
    tag = true
  }
  return tag
}

function mapAndSetResult (special, res, runtime) {
  return {
    "type": "object",
    "subtype": special.toLowerCase(),
    "className": special,
    "description": `${special}(${res.size})`,
    "objectId": runtime.setItemObjectId(res.__objectId__, res),
    "preview": {
        "type": "object",
        "subtype": special.toLowerCase(),
        "description": `${special}(${res.size})`,
        "overflow": false,
        "properties": [
            {
                "name": "size",
                "type": "number",
                "value": res.size
            }
        ]
    }
  }
}

function arrayBufferResult (res, runtime) {
  return {
    "type": "object",
    "subtype": "arraybuffer",
    "className": "ArrayBuffer",
    "description": `ArrayBuffer(${res.byteLength})`,
    "objectId": runtime.setItemObjectId(res.__objectId__, res),
    "preview": {
        "type": "object",
        "subtype": "arraybuffer",
        "description": `ArrayBuffer(${res.byteLength})`,
        "overflow": false,
        "properties": [
            {
                "name": "byteLength",
                "type": "number",
                "value": res.byteLength
            }
        ]
    }
  }
}

export default {
  getSpecialType,
  unsupportedObjects,
  mapAndSetResult,
  arrayBufferResult
}
