import { postDebuggerMsg } from './index'
import DomStyleSheets from './DomStyleSheets'
import _highlight from './HighlightHelp'
//     -------------------------------   DomRetainer -------------------------------
class DomRetainer {
  constructor() {
    (this.NODE_COUNTER = 1),
      (this.NODE_ID_DOM_MAP = {}),
      (this.DOM_WEAK_MAP = new WeakMap());
  }
  getNodeIdFromMap(t) {
    if (t) return this.DOM_WEAK_MAP.get(t);
  }
  getNodeId(t) {
    if (!t) return;
    let e = this.DOM_WEAK_MAP.get(t);
    return (
      e ||
        ((e = this.NODE_COUNTER++),
        t.setAttribute && t.setAttribute("wx:nodeid", e),
        (this.NODE_ID_DOM_MAP[e] = t),
        this.DOM_WEAK_MAP.set(t, e)),
      e
    );
  }
  getDOMByNodeId(t) {
    return this.NODE_ID_DOM_MAP[t];
  }
  removeDOM(t) {
    const e = this.DOM_WEAK_MAP.get(t);
    return (
      e && (delete this.NODE_ID_DOM_MAP[e], this.DOM_WEAK_MAP.delete(t)), e
    );
  }
}

const requestChildNodes = (nodeId, depth = 1) => {
  const domNode = domRetainer.getDOMByNodeId(nodeId);
  const children = [];
  let childNodeCount = 0;

  if (domNode && domNode.childNodes) {
    for (let idx = 0, n = domNode.childNodes.length; idx < n; idx++) {
      const childrenNode = domNode.childNodes[idx];
      if (
        "STYLE" !== childrenNode.tagName &&
        "SCRIPT" !== childrenNode.tagName
      ) {
        childNodeCount++;
        if (depth < 0 || depth >= 1) {
          children.push(formatNode(childrenNode, depth - 1));
        }
      }
    }
  }
  return {
    children: children,
    childNodeCount: childNodeCount,
  };
};

const formatNode = (domNode, depth = -1) => {
  const id = domRetainer.getNodeId(domNode);
  let parentId;

  if (domNode.parentElement) {
    parentId = domRetainer.getNodeId(domNode.parentElement);
  }

  const res = {
    parentId: parentId,
    nodeId: id,
    localName: domNode.localName,
    nodeName: domNode.nodeName,
    nodeType: domNode.nodeType,
    nodeValue: domNode.nodeValue || "",
    attributes: [],
  };

  if (domNode.attributes) {
    for (let e = 0, r = domNode.attributes.length; e < r; e++) {
      const r = domNode.attributes[e];
      res.attributes.push(r.name), res.attributes.push(r.value);
    }
  }

  const { children, childNodeCount, nodeValue } = requestChildNodes(id, depth);
  res.childNodeCount = childNodeCount;
  
  if (children.length) {
    res.children = children
  }
  if (nodeValue) {
    res.nodeValue = nodeValue
  }
  return res;
};

const domRetainer = new DomRetainer();
const domStyleSheets = new DomStyleSheets()
//  -------------------------------   DomRetainer -------------------------------


let isObserved = false;
let observer = null;

const attributesMutation = (mutation) => {
  const targetId = domRetainer.getNodeIdFromMap(mutation.target)
  if (!targetId) return
  const attributeValue = mutation.target.getAttribute(mutation.attributeName)
  if (attributeValue === null) {
    emitEvent('DOM.attributeRemoved', {
      nodeId: targetId,
      name: mutation.attributeName
    })
  } else {
    emitEvent('DOM.attributeModified', {
      nodeId: targetId,
      name: mutation.attributeName,
      value: attributeValue
    })
  }
}

const childListMutation = (mutation) => {
  const targetId = domRetainer.getNodeIdFromMap(mutation.target)
  if (!targetId) return

  let previousId = domRetainer.getNodeIdFromMap(mutation.previousSibling)

  mutation.removedNodes && mutation.removedNodes.forEach(domNode => {

    if (domNode.nodeName === 'BODY') {
      emitEvent("DOM.documentUpdated", {})
    }

    const nodeId = domRetainer.removeDOM(domNode);
    if (nodeId) {
      emitEvent('DOM.childNodeRemoved', {
        parentNodeId: targetId,
        nodeId
      })
    }
  })

  mutation.addedNodes && mutation.addedNodes.forEach(domNode => {
    const node = formatNode(domNode, 1)
    emitEvent('DOM.childNodeInserted', {
      parentNodeId: targetId,
      previousNodeId: previousId, // TODO previousId 机制 可能有问题
      node
    })
    previousId = node.nodeId
  })

}

const characterDataMutation = (mutation) => {
  const targetId = domRetainer.getNodeIdFromMap(mutation.target)
  if (!targetId) return
  setTimeout(() => {
    emitEvent('DOM.characterDataModified', {
      nodeId: targetId,
      name: mutation.target.nodeValue
    })
  }, 10);
}

const handleGetDocument = (reqPayload) => {
  const documentId = domRetainer.getNodeId(document);
  if (!isObserved) {
    observer = new MutationObserver(function (mutationList) {
      mutationList.forEach((mutation) => {
        switch (mutation.type) {
          case "attributes":
            attributesMutation(mutation);
            break;
          case "childList":
            childListMutation(mutation);
            break;
          case "characterData":
            characterDataMutation(mutation)
        }
      });
    });
    observer.observe(document, {
      attributes: true,
      childList: true,
      characterData: true,
      subtree: true,
      attributeOldValue: true,
      characterDataOldValue: true,
    });
    isObserved = true;
  }
  const result = {
    root: formatNode(document, 2)
  };
  sendRes(reqPayload, result);
};

const handleRequestChildNodes = (reqPayload) => {
  const { nodeId, depth = 1 } = reqPayload.params;
  const res = requestChildNodes(nodeId, depth)
  emitEvent('DOM.setChildNodes', {
    parentId: nodeId,
    nodes: res.children
  })
  setTimeout(() => {
    sendRes(reqPayload, {});
  }, 0);
}

const handleQuerySelector = (reqPayload) => {
  const { nodeId, selector } = reqPayload.params;
  const node = domRetainer.getDOMByNodeId(nodeId);
  if (node) {
    const targetNode = node.querySelector(selector);
    if (targetNode) {
      sendRes(reqPayload, {
        nodeId: domRetainer.getNodeId(targetNode),
      });
    }
  }
};

function handleCssEnable(reqPayload) {
  postDebuggerMsg('dom', {
    id: reqPayload.id,
    result: {}
  })
  emitEvent('CSS.styleSheetAdded', {
    endColumn: 0,
    endLine: 0,
    sourceURL: "",
    startColumn: 0,
    startLine: 0,
    styleSheetId: "style_0"
  })
}

function getMatchedStylesForNode (reqPayload) {
  const { nodeId: e } = reqPayload.params
  const r = domRetainer.getDOMByNodeId(e)
  const result = {
    cssKeyframesRules: [],
    inherited: [],
    pseudoElements: [],
    matchedCSSRules: domStyleSheets.getMatchedStylesForNode(r),
    inlineStyle: domStyleSheets.getInlineStyle(r, domRetainer)
  }
  sendRes(reqPayload, result)
}

function getComputedStyleForNode (reqPayload) {
  const { nodeId: e } = reqPayload.params
  const r = domRetainer.getDOMByNodeId(e)
  sendRes(reqPayload, {
    computedStyle: domStyleSheets.getComputedStyleForNode(r)
  })
}

function setStyleTexts (reqPayload) {
  for (var { edits: e } = reqPayload.params, r = 0, n = e.length; r < n; r++) {
    domStyleSheets.setStyleTexts(e[r], domRetainer)
  }
  return {}
}

function hideHighlight (reqPayload) {
  _highlight.hide()
}

function highlightNode (reqPayload) {
  const nodeId = reqPayload.params.nodeId
  const o = domRetainer.getDOMByNodeId(nodeId)
  o && _highlight.show(o)
}

const messageHandler = {
  "DOM.getDocument": handleGetDocument,
  "DOM.requestChildNodes": handleRequestChildNodes,
  "DOM.querySelector": handleQuerySelector,
  "CSS.enable": handleCssEnable,
  'CSS.getMatchedStylesForNode': getMatchedStylesForNode,
  'CSS.getComputedStyleForNode': getComputedStyleForNode,
  'CSS.setStyleTexts': setStyleTexts,
  'Overlay.highlightNode': highlightNode,
  'Overlay.hideHighlight': hideHighlight
}

export const domCdpHandler = (cdp) => {
  if (messageHandler[cdp.method]) {
    messageHandler[cdp.method](cdp)
    return
  }
  if (cdp.id) {
    postDebuggerMsg('dom', {
      id: cdp.id,
      result: {}
    })
  }
}

const sendMsg = (data) => {
  postDebuggerMsg('dom', data)
};

const sendRes = (reqPayload, result = {}) => {
  sendMsg({
    id: reqPayload.id,
    result: result,
  });
};

const emitEvent = (method, params) => {
  sendMsg({
    method,
    params
  });
}